import logging

from django.http import HttpRequest
from django.views import View
from rest_framework import permissions

log = logging.getLogger(__name__)


class IsUserRoleGenericPermission(permissions.BasePermission):
    """ Generic class that encapsulates how to identify someone
    as a member of a user Group """

    def has_permission(self, request: HttpRequest, view: View) -> bool:
        """ required by BasePermission. Check if user is instance of any
        of the models provided in class' models attribute """
        assert self.roles is not None, (
            "'%s' has to include a `roles` attribute"
            % self.__class__.__name__
        )

        user = request.user
        is_authorized = user.is_superuser or (user.is_authenticated and
                                              user.role in self.roles)

        if not is_authorized:
            log.warn('User "%s" has no permission to view %s',
                     user.username, view.__class__.__name__)

        return is_authorized


class IsStudent(IsUserRoleGenericPermission):
    """ Has student permissions """
    roles = ('Student', )


class IsReviewer(IsUserRoleGenericPermission):
    """ Has reviewer permissions """
    roles = ('Reviewer', )


class IsTutor(IsUserRoleGenericPermission):
    """ Has tutor permissions """
    roles = ('Tutor', )


class IsTutorOrReviewer(IsUserRoleGenericPermission):
    """ Has tutor or reviewer permissions """
    roles = ('Tutor', 'Reviewer')
