import logging

from rest_framework import serializers

from core import models
from util.factories import GradyUserFactory

from .generic import DynamicFieldsModelSerializer

log = logging.getLogger(__name__)
user_factory = GradyUserFactory()


class ExamSerializer(DynamicFieldsModelSerializer):

    class Meta:
        model = models.ExamType
        fields = ('pk', 'module_reference', 'total_score',
                  'pass_score', 'pass_only',)


class TestSerializer(DynamicFieldsModelSerializer):

    class Meta:
        model = models.Test
        fields = ('pk', 'name', 'label', 'annotation')


class SubmissionTypeListSerializer(DynamicFieldsModelSerializer):

    class Meta:
        model = models.SubmissionType
        fields = ('pk', 'name', 'full_score')


class SubmissionTypeSerializer(SubmissionTypeListSerializer):

    class Meta:
        model = models.SubmissionType
        fields = ('pk',
                  'name',
                  'full_score',
                  'description',
                  'solution',
                  'programming_language')


class TutorSerializer(DynamicFieldsModelSerializer):
    feedback_created = serializers.SerializerMethodField()
    feedback_validated = serializers.SerializerMethodField()

    @staticmethod
    def _get_completed_assignments(obj):
        return models.TutorSubmissionAssignment.objects.filter(
            is_done=True,
            subscription__owner=obj,
        )

    def get_feedback_created(self, obj):
        return self._get_completed_assignments(obj).filter(
            subscription__feedback_stage=models.SubmissionSubscription.FEEDBACK_CREATION  # noqa
        ).count()

    def get_feedback_validated(self, obj):
        return self._get_completed_assignments(obj).filter(
            subscription__feedback_stage=models.SubmissionSubscription.FEEDBACK_VALIDATION  # noqa
        ).count()

    def create(self, validated_data) -> models.UserAccount:
        log.info("Crating tutor from data %s", validated_data)
        return user_factory.make_tutor(
            username=validated_data['username'])

    class Meta:
        model = models.UserAccount
        fields = ('pk', 'username', 'feedback_created', 'feedback_validated')
