from rest_framework import serializers

from core.models import StudentInfo
from core.serializers import DynamicFieldsModelSerializer, ExamSerializer
from core.serializers.submission import (SubmissionListSerializer,
                                         SubmissionNoTextFieldsSerializer)


class StudentInfoSerializer(DynamicFieldsModelSerializer):
    name = serializers.ReadOnlyField(source='user.fullname')
    matrikel_no = serializers.ReadOnlyField(source='user.matrikel_no')
    exam = ExamSerializer()
    submissions = SubmissionListSerializer(many=True)

    class Meta:
        model = StudentInfo
        fields = ('pk', 'name', 'user', 'matrikel_no', 'exam', 'submissions')


class StudentInfoSerializerForListView(DynamicFieldsModelSerializer):
    name = serializers.ReadOnlyField(source='user.fullname')
    user = serializers.ReadOnlyField(source='user.username')
    exam = serializers.ReadOnlyField(source='exam.module_reference')
    submissions = SubmissionNoTextFieldsSerializer(many=True)

    class Meta:
        model = StudentInfo
        fields = ('pk', 'name', 'user', 'exam', 'submissions', 'matrikel_no')
