from django.urls import path
from rest_framework.routers import DefaultRouter

from core import views

# Create a router and register our viewsets with it.
router = DefaultRouter()
router.register('student', views.StudentReviewerApiViewSet,
                base_name='student')
router.register('examtype', views.ExamApiViewSet)
router.register('feedback', views.FeedbackApiView)
router.register('feedback-comment', views.FeedbackCommentApiView)
router.register('submission', views.SubmissionViewSet,
                base_name='submission')
router.register('submissiontype', views.SubmissionTypeApiView)
router.register('tutor', views.TutorApiViewSet, base_name='tutor')
router.register('subscription', views.SubscriptionApiViewSet,
                base_name='subscription')
router.register('assignment', views.AssignmentApiViewSet)
router.register('statistics', views.StatisticsEndpoint, base_name='statistics')

# regular views that are not viewsets
regular_views_urlpatterns = [
    path('student-page/',
         views.StudentSelfApiView.as_view(),
         name='student-page'),
    path('student-submissions/',
         views.StudentSelfSubmissionsApiView.as_view(),
         name='student-submissions'),
    path('user-role/', views.get_user_role, name='user-role'),
    path('jwt-time-delta/',
         views.get_jwt_expiration_delta,
         name='jwt-time-delta'),
    path('export/csv/', views.StudentCSVExport.as_view(), name='export-csv'),
]

urlpatterns = [
    *router.urls,
    *regular_views_urlpatterns,
]
