const getters = {
  getSubscriptionsGroupedByType (state, getters) {
    let subscriptions = {
      'random': [],
      'student': [],
      'exam': [],
      'submission_type': []
    }
    Object.entries(state.subscriptions).forEach(([id, subscription]) => {
      subscriptions[subscription.query_type].push(subscription)
    })
    // sort the resulting arrays in subscriptions lexicographically by their query_keys
    Object.entries(subscriptions).forEach(([id, arr]) => {
      if (arr.length > 1 && arr[0].hasOwnProperty('query_key')) {
        arr.sort((subA, subB) => {
          const subALower = getters.resolveSubscriptionKeyToName(subA).toLowerCase()
          const subBLower = getters.resolveSubscriptionKeyToName(subB).toLowerCase()
          if (subALower < subBLower) {
            return -1
          } else if (subALower > subBLower) {
            return 1
          } else {
            return 0
          }
        })
      }
    })
    return subscriptions
  },
  corrected (state) {
    return state.statistics.submission_type_progress.every(progress => {
      return progress.percentage === 100
    })
  },
  resolveSubscriptionKeyToName: state => subscription => {
    if (subscription.query_type === 'random') {
      return 'Active'
    } else if (subscription.query_type === 'exam') {
      return state.examTypes[subscription.query_key].module_reference
    } else if (subscription.query_type === 'submission_type') {
      return state.submissionTypes[subscription.query_key].name
    } else if (subscription.query_type === 'student') {
      const studentName = state.students[subscription.query_key]
      return studentName ? studentName.name : 'Student'
    }
  },
  getSubmission: state => pk => {
    return state.submissions[pk]
  },
  getFeedback: state => pk => {
    return state.feedback[pk]
  },
  getSubmissionType: state => pk => {
    return state.submissionTypes[pk]
  }
}

export default getters
