import Vue from 'vue'

import {initialState} from '@/store/store'

export const mut = Object.freeze({
  ADD_ASSIGNMENT_TO_SUBSCRIPTION_QUEUE: 'ADD_ASSIGNMENT_TO_SUBSCRIPTION_QUEUE',
  DELETE_ASSIGNMENT_FROM_SUBSCRIPTION_QUEUE: 'DELETE_ASSIGNMENT_FROM_SUBSCRIPTION_QUEUE',
  SET_ASSIGNMENT: 'SET_ASSIGNMENT',
  SET_SUBSCRIPTIONS: 'SET_SUBSCRIPTIONS',
  SET_SUBSCRIPTION: 'SET_SUBSCRIPTION',
  DELETE_SUBSCRIPTION: 'DELETE_SUBSCRIPTION',
  SET_LAST_INTERACTION: 'SET_LAST_INTERACTION',
  SET_EXAM_TYPES: 'SET_EXAM_TYPES',
  SET_STUDENTS: 'SET_STUDENTS',
  SET_STUDENT: 'SET_STUDENT',
  SET_TUTORS: 'SET_TUTORS',
  SET_SUBMISSION: 'SET_SUBMISSION',
  SET_ALL_FEEDBACK: 'SET_ALL_FEEDBACK',
  SET_STATISTICS: 'SET_STATISTICS',
  SET_FEEDBACK: 'SET_FEEDBACK',
  MAP_FEEDBACK_OF_SUBMISSION_TYPE: 'MAP_FEEDBACK_OF_SUBMISSION_TYPE',
  UPDATE_SUBMISSION_TYPE: 'UPDATE_SUBMISSION_TYPE',
  UPDATE_SUBMISSION: 'UPDATE_SUBMISSION',
  RESET_STATE: 'RESET_STATE'
})

const mutations = {
  [mut.SET_EXAM_TYPES] (state, examTypes) {
    state.examTypes = examTypes.reduce((acc, curr) => {
      acc[curr.pk] = curr
      return acc
    }, {})
  },
  [mut.SET_ASSIGNMENT] (state, assignment) {
    Vue.set(state.assignments, assignment.pk, assignment)
  },
  [mut.SET_SUBSCRIPTIONS] (state, subscriptions) {
    state.subscriptions = subscriptions.reduce((acc, curr) => {
      acc[curr['pk']] = curr
      return acc
    }, {})
    for (let subscription of Object.values(state.subscriptions)) {
      for (let assignment of subscription.assignments) {
        if (assignment.feedback) {
          Vue.set(assignment.feedback, 'feedback_stage_for_user', subscription.feedback_stage)
        }
      }
    }
  },
  [mut.SET_SUBSCRIPTION] (state, subscription) {
    Vue.set(state.subscriptions, subscription.pk, subscription)
    for (let assignment of subscription.assignments) {
      if (assignment.feedback) {
        Vue.set(assignment.feedback, 'feedback_stage_for_user', subscription.feedback_stage)
      }
    }
  },
  [mut.DELETE_SUBSCRIPTION] (state, {pk}) {
    Vue.delete(state.subscriptions, pk)
  },
  [mut.SET_STUDENTS] (state, students) {
    state.students = students.reduce((acc, curr) => {
      acc[curr.pk] = curr
      return acc
    }, {})
  },
  [mut.SET_STUDENT] (state, student) {
    Vue.set(state.students, student.pk, {
      ...state.students[student.pk],
      ...student
    })
  },
  [mut.SET_TUTORS] (state, tutors) {
    state.tutors = tutors
  },
  [mut.SET_SUBMISSION] (state, submission) {
    Vue.set(state.submissions, submission.pk, submission)
  },
  [mut.SET_ALL_FEEDBACK] (state, feedbackList) {
    state.feedback = feedbackList.reduce((acc, curr) => {
      acc[curr.pk] = curr
      return acc
    }, {})
  },
  [mut.SET_FEEDBACK] (state, feedback) {
    Vue.set(state.feedback, feedback.pk, {
      ...state.feedback[feedback.pk],
      ...feedback,
      of_submission_type: state.submissionTypes[feedback['of_submission_type']]
    })
  },
  [mut.SET_STATISTICS] (state, statistics) {
    state.statistics = {
      ...state.statistics,
      ...statistics
    }
  },
  [mut.MAP_FEEDBACK_OF_SUBMISSION_TYPE] (state) {
    Object.values(state.feedback).forEach(feedback => {
      const submissionType = state.submissionTypes[feedback['of_submission_type']]
      if (submissionType) {
        feedback['of_submission_type'] = submissionType
      }
    })
  },
  [mut.UPDATE_SUBMISSION_TYPE] (state, submissionType) {
    const updatedSubmissionType = {
      ...state.submissionTypes[submissionType.pk],
      ...submissionType
    }
    Vue.set(state.submissionTypes, submissionType.pk, updatedSubmissionType)
  },
  [mut.UPDATE_SUBMISSION] (state, {submissionPk, payload, key}) {
    Vue.set(state.submissions[submissionPk], key, payload)
  },
  [mut.ADD_ASSIGNMENT_TO_SUBSCRIPTION_QUEUE] (state, {assignment}) {
    let subscription = state.subscriptions[assignment.subscription]
    subscription['assignments'].push(assignment)
  },
  [mut.DELETE_ASSIGNMENT_FROM_SUBSCRIPTION_QUEUE] (state, {subscription}) {
    subscription.assignments.shift()
  },

  [mut.SET_LAST_INTERACTION] (state) {
    state.lastAppInteraction = Date.now()
  },
  [mut.RESET_STATE] (state) {
    Object.assign(state, initialState())
  }
}

export default mutations
